from collections import UserDict
from typing import Any, Iterable, final

from .core import BasePattern

@final
class Patterns(UserDict[Any, BasePattern]):
    name: str
    def __init__(self, name: str): ...
    def set(
        self, target: BasePattern[Any, Any, Any], alias: str | None = None, cover: bool = True, no_alias=False
    ):
        """
        增加可使用的类型转换器

        Args:
            target: 设置的表达式
            alias: 目标类型的别名
            cover: 是否覆盖已有的转换器
            no_alias: 是否不使用目标类型自带的别名
        """
        ...

    def sets(self, patterns: Iterable[BasePattern[Any, Any, Any]], cover: bool = True, no_alias=False): ...
    def merge(self, patterns: dict[str, BasePattern[Any, Any, Any]], no_alias=False): ...
    def remove(self, origin_type: type, alias: str | None = None): ...

def create_local_patterns(
    name: str,
    data: dict[Any, BasePattern[Any, Any, Any]] | None = None,
    set_current: bool = True,
) -> Patterns:
    """
    新建一个本地表达式组

    Args:
        name: 组名
        data: 可选的初始内容
        set_current: 是否设置为 current
    """
    ...

def switch_local_patterns(name: str) -> None: ...
def reset_local_patterns() -> None: ...
def local_patterns() -> Patterns: ...
def global_patterns() -> Patterns: ...
def all_patterns() -> Patterns:
    """获取 global 与 local 的合并表达式组"""



from __future__ import annotations

from collections.abc import Mapping as ABCMap
from collections.abc import MutableMapping as ABCMuMap
from collections.abc import MutableSequence as ABCMuSeq
from collections.abc import MutableSet as ABCMuSet
from collections.abc import Sequence as ABCSeq
from collections.abc import Set as ABCSet
from contextlib import suppress
from copy import deepcopy
import inspect
from types import FunctionType, LambdaType, MethodType
from typing import Any, ForwardRef, Literal, TypeVar, Union, runtime_checkable
from typing_extensions import Annotated, get_args, get_origin

from tarina.lang import lang

from .base import (
    ANY,
    NONE,
    DirectPattern,
    ForwardRefPattern,
    MappingPattern,
    RegexPattern,
    SequencePattern,
    SwitchPattern,
    UnionPattern,
)
from .context import all_patterns
from .core import BasePattern, MatchMode
from .util import CGenericAlias, CUnionType, GenericAlias, RawStr, TPattern

_Contents = (Union, CUnionType, Literal)


def _generic_parser(item: GenericAlias, extra: str) -> BasePattern:  # type: ignore
    origin = get_origin(item)
    if origin is Annotated:
        org, *meta = get_args(item)
        if not isinstance(_o := parser(org, extra), BasePattern):  # type: ignore  # pragma: no cover
            raise TypeError(_o)
        _arg = deepcopy(_o)
        _arg.alias = al[-1] if (al := [i for i in meta if isinstance(i, str)]) else _arg.alias
        _arg.validators.extend(i for i in meta if callable(i))
        return _arg
    if origin in _Contents:
        _args = {parser(t, extra) for t in get_args(item)}
        return (_args.pop() if len(_args) == 1 else UnionPattern(_args)) if _args else ANY
    if origin in (dict, ABCMap, ABCMuMap):
        if args := get_args(item):
            return MappingPattern(
                arg_key=parser(args[0], "ignore"),
                arg_value=parser(args[1], "allow"),
            )
        return MappingPattern(ANY, ANY)  # pragma: no cover
    _args = parser(args[0], "allow") if (args := get_args(item)) else ANY
    if origin in (ABCMuSeq, list):
        return SequencePattern(list, _args)
    if origin in (ABCSeq, tuple):
        return SequencePattern(tuple, _args)
    if origin in (ABCMuSet, ABCSet, set):
        return SequencePattern(set, _args)
    return BasePattern(mode=MatchMode.KEEP, origin=origin, alias=f"{repr(item).split('.')[-1]}", accepts=origin)  # type: ignore


def _typevar_parser(item: TypeVar):
    return BasePattern(mode=MatchMode.KEEP, origin=Any, alias=f"{item}"[1:], accepts=item)  # type: ignore


def _protocol_parser(item: type):
    if not getattr(item, "_is_runtime_protocol", True):  # pragma: no cover
        item = runtime_checkable(deepcopy(item))  # type: ignore
    return BasePattern(mode=MatchMode.KEEP, alias=f"{item}", accepts=item)


def parser(item: Any, extra: str = "allow") -> BasePattern:
    """对数类型的检查， 将一般数据类型转为 BasePattern 或者特殊类型"""
    if isinstance(item, BasePattern):
        return item
    with suppress(TypeError):
        if item and (pat := all_patterns().get(item, None)):
            return pat
    if isinstance(item, (GenericAlias, CGenericAlias, CUnionType)):
        return _generic_parser(item, extra)
    if isinstance(item, TypeVar):
        return _typevar_parser(item)
    if getattr(item, "_is_protocol", False):
        return _protocol_parser(item)
    if isinstance(item, (FunctionType, MethodType, LambdaType)):
        if len((sig := inspect.signature(item)).parameters) not in (1, 2):  # pragma: no cover
            raise TypeError(f"{item} can only accept 1 or 2 argument")
        anno = list(sig.parameters.values())[-1].annotation
        return BasePattern(
            accepts=Any if anno == inspect.Signature.empty else anno,  # type: ignore
            origin=(Any if sig.return_annotation == inspect.Signature.empty else sig.return_annotation),  # type: ignore
            converter=item if len(sig.parameters) == 2 else lambda _, x: item(x),
            mode=MatchMode.TYPE_CONVERT,
        )
    if isinstance(item, TPattern):  # type: ignore
        return RegexPattern(item.pattern, alias=f"'{item.pattern}'")
    if isinstance(item, str):
        if item.startswith("re:"):
            pat = item[3:]
            return BasePattern(pat, MatchMode.REGEX_MATCH, alias=f"'{pat}'")
        if item.startswith("rep:"):
            pat = item[4:]
            return RegexPattern(pat, alias=f"'{pat}'")
        if "|" in item:
            names = item.split("|")
            return UnionPattern(all_patterns().get(i, i) for i in names if i)
        return DirectPattern(item, alias=f"'{item}'")
    if isinstance(item, RawStr):
        return DirectPattern(item.value, alias=f"'{item.value}'")
    if isinstance(item, (list, tuple, set, ABCSeq, ABCMuSeq, ABCSet, ABCMuSet)):  # Args[foo, [123, int]]
        return UnionPattern(map(lambda x: parser(x) if inspect.isclass(x) else x, item))
    if isinstance(item, (dict, ABCMap, ABCMuMap)):
        return SwitchPattern(dict(item))
    if isinstance(item, ForwardRef):
        return ForwardRefPattern(item)
    if item is None or type(None) == item:
        return NONE
    if extra == "ignore":
        return ANY
    elif extra == "reject":
        raise TypeError(lang.require("nepattern", "validate_reject").format(target=item))
    return BasePattern.of(item) if inspect.isclass(item) else BasePattern.on(item)


__all__ = ["parser"]



from __future__ import annotations

from enum import Enum, IntEnum
import re
from typing import Any, Callable, Generic, Literal, TypeVar, overload
from typing_extensions import NoReturn, Self

from tarina import Empty

from .base import DirectPattern, DirectTypePattern
from .util import TPattern

class MatchMode(IntEnum):
    """参数表达式匹配模式"""

    VALUE_OPERATE = 4
    """传入值进行操作"""
    REGEX_CONVERT = 3
    """正则匹配并转换"""
    TYPE_CONVERT = 2
    """传入值直接转换"""
    REGEX_MATCH = 1
    """正则匹配"""
    KEEP = 0
    """保持传入值"""

class ResultFlag(str, Enum):
    """参数表达式验证结果标识符"""

    VALID = "valid"
    "验证通过"
    ERROR = "error"
    "验证失败"
    DEFAULT = "default"
    "默认值"

T = TypeVar("T")
T1 = TypeVar("T1")
TInput = TypeVar("TInput", covariant=True)
TInput1 = TypeVar("TInput1")
TInput2 = TypeVar("TInput2")
TInput3 = TypeVar("TInput3")
TInput4 = TypeVar("TInput4")
TOrigin = TypeVar("TOrigin")
TOrigin1 = TypeVar("TOrigin1")
TVOrigin = TypeVar("TVOrigin")
TDefault = TypeVar("TDefault")
TVRF = TypeVar("TVRF", bound=ResultFlag)
TMM = TypeVar("TMM", bound=MatchMode)

class ValidateResult(Generic[TVOrigin, TVRF]):
    """参数表达式验证结果"""

    flag: TVRF
    _value: TVOrigin | type[Empty]
    _error: Exception | type[Empty]

    def __init__(
        self,
        value: TVOrigin | type[Empty] = Empty,
        error: Exception | type[Empty] = Empty,
        flag: TVRF = ResultFlag.VALID,
    ): ...
    @overload
    def value(self: ValidateResult[Any, Literal[ResultFlag.ERROR]]) -> NoReturn: ...
    @overload
    def value(self: ValidateResult[TVOrigin, TVRF]) -> TVOrigin: ...
    @overload
    def error(self: ValidateResult[Any, Literal[ResultFlag.VALID]]) -> None: ...
    @overload
    def error(self: ValidateResult[Any, Literal[ResultFlag.ERROR]]) -> Exception: ...
    @overload
    def error(self: ValidateResult[Any, Literal[ResultFlag.DEFAULT]]) -> None: ...
    @overload
    def error(self: ValidateResult[TVOrigin, TVRF]) -> Exception | None: ...
    @property
    def success(self) -> bool: ...
    @property
    def failed(self) -> bool: ...
    @property
    def or_default(self) -> bool: ...
    @overload
    def step(self, other: type[T]) -> T: ...
    @overload
    def step(self, other: BasePattern[T, TVOrigin, MatchMode]) -> ValidateResult[T, TVRF]: ...
    @overload
    def step(self, other: Callable[[TVOrigin], T]) -> T: ...
    @overload
    def step(self, other: Any) -> Self: ...
    @overload
    def __rshift__(self, other: BasePattern[T, TVOrigin, MatchMode]) -> ValidateResult[T, TVRF]: ...
    @overload
    def __rshift__(self, other: type[T]) -> T: ...
    @overload
    def __rshift__(self, other: Callable[[TVOrigin], T]) -> T: ...
    @overload
    def __rshift__(self, other: Any) -> Self: ...
    @overload
    def __bool__(
        self: (
            ValidateResult[TVOrigin, Literal[ResultFlag.VALID]]
            | ValidateResult[TVOrigin, Literal[ResultFlag.DEFAULT]]
        )
    ) -> Literal[True]: ...
    @overload
    def __bool__(self: ValidateResult[TVOrigin, Literal[ResultFlag.ERROR]]) -> Literal[False]: ...
    def __repr__(self): ...

_MATCHES: dict[MatchMode, Callable[[Any], Callable[[Any, Any], Any]]] = {}

class BasePattern(Generic[TOrigin, TInput, TMM]):
    """对参数类型值的包装"""

    regex_pattern: TPattern  # type: ignore
    pattern: str
    mode: MatchMode
    converter: Callable[[BasePattern[TOrigin, TInput, TMM], Any], TOrigin | None]
    validators: list[Callable[[TOrigin], bool]]

    origin: type[TOrigin]
    alias: str | None
    previous: BasePattern | None
    accept: Callable[[Any], bool]
    _repr: str
    _hash: int
    _accepts: type[Any] | tuple[type[Any], ...]
    _pattern_accepts: BasePattern | None

    @overload
    def __init__(
        self: BasePattern[Any, Any, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        origin: Any = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[T], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1, TInput1, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        accepts: type[TInput1],
        origin: Any = None,
        alias: str | None = None,
        previous: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TInput1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1, TInput1, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        addition_accepts: BasePattern[Any, TInput1, Any],
        origin: Any = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        validators: list[Callable[[TInput1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1 | TInput2, TInput1 | TInput2, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        accepts: type[TInput1],
        addition_accepts: BasePattern[Any, TInput2, Any],
        origin: Any = None,
        alias: str | None = None,
        previous: None = None,
        validators: list[Callable[[TInput1 | TInput2], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1, TInput1, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        accepts: type[TInput1],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        origin: Any = None,
        alias: str | None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TInput1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1, TInput1, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        addition_accepts: BasePattern[Any, TInput1, Any],
        origin: Any = None,
        alias: str | None = None,
        accepts: None = None,
        validators: list[Callable[[TInput1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TInput1 | TInput2, TInput1 | TInput2, Literal[MatchMode.KEEP]],
        *,
        mode: Literal[MatchMode.KEEP],
        accepts: type[TInput1],
        previous: BasePattern[TInput1 | TInput2, TInput1 | TInput2, Literal[MatchMode.VALUE_OPERATE]],
        addition_accepts: BasePattern[Any, TInput2, Any],
        origin: Any = None,
        alias: str | None = None,
        validators: list[Callable[[TInput1 | TInput2], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_MATCH],
        origin: type[str] = str,
        converter: None = None,
        alias: str | None = None,
        previous: None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[str], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_MATCH],
        previous: BasePattern[str, str, Literal[MatchMode.VALUE_OPERATE]],
        origin: type[str] = str,
        converter: None = None,
        alias: str | None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[str], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[str, str | TInput1, Literal[MatchMode.REGEX_MATCH]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_MATCH],
        previous: BasePattern[str, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        origin: type[str] = str,
        converter: None = None,
        alias: str | None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[str], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, str | TOrigin1, Literal[MatchMode.REGEX_CONVERT]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_CONVERT],
        origin: type[TOrigin1],
        converter: (
            Callable[
                [BasePattern[TOrigin1, str | TOrigin1, Literal[MatchMode.REGEX_CONVERT]], re.Match[str]],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        previous: None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, str | TOrigin1 | TInput1, Literal[MatchMode.REGEX_CONVERT]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_CONVERT],
        origin: type[TOrigin1],
        previous: (
            BasePattern[str | TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[str, TInput1, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]]
        ),
        converter: (
            Callable[
                [
                    BasePattern[TOrigin1, str | TOrigin1 | TInput1, Literal[MatchMode.REGEX_CONVERT]],
                    re.Match[str],
                ],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, str | TOrigin1, Literal[MatchMode.REGEX_CONVERT]],
        pattern: str,
        mode: Literal[MatchMode.REGEX_CONVERT],
        origin: type[TOrigin1],
        previous: (
            BasePattern[str | TOrigin1, str | TOrigin1, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[str, str, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TOrigin1, TOrigin1, Literal[MatchMode.VALUE_OPERATE]]
        ),
        converter: (
            Callable[
                [BasePattern[TOrigin1, str | TOrigin1, Literal[MatchMode.REGEX_CONVERT]], re.Match[str]],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: type[str] = str,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        converter: Callable[
            [BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]], Any], TOrigin1 | None
        ],
        origin: None = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
        ],
        origin: None = None,
        alias: str | None = None,
        previous: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        addition_accepts: BasePattern[Any, TInput1, Any],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
        ],
        origin: None = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        addition_accepts: BasePattern[Any, TInput2, Any],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput2],
            TOrigin1 | None,
        ],
        origin: None = None,
        alias: str | None = None,
        previous: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        converter: (
            Callable[[BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]], Any], TOrigin1 | None] | None
        ) = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
            ]
            | None
        ) = None,
        alias: str | None = None,
        previous: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        addition_accepts: BasePattern[Any, TInput1, Any],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
            ]
            | None
        ) = None,
        alias: str | None = None,
        previous: None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        addition_accepts: BasePattern[Any, TInput2, Any],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput2],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        previous: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        previous: BasePattern[TInput1, Any, Literal[MatchMode.TYPE_CONVERT]],
        converter: Callable[
            [BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]], TInput1 | Any], TOrigin1 | None
        ],
        alias: str | None = None,
        origin: None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        previous: BasePattern[TInput1, Any, Literal[MatchMode.TYPE_CONVERT]],
        converter: (
            Callable[
                [BasePattern[TOrigin1, Any, Literal[MatchMode.TYPE_CONVERT]], TInput1 | Any], TOrigin1 | None
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
        ],
        alias: str | None = None,
        origin: None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TOrigin1 | TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        previous: (
            BasePattern[TOrigin1 | TInput1, TOrigin1 | TInput1, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TOrigin1, TOrigin1, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]]
        ),
        converter: (
            Callable[
                [BasePattern[TOrigin1, TOrigin1 | TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        previous: BasePattern[TInput4 | TInput1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput4],
            TOrigin1 | None,
        ],
        alias: str | None = None,
        origin: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        previous: BasePattern[TInput4 | TInput1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput4],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
        ],
        alias: str | None = None,
        origin: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
            ]
            | None
        ) = None,
        alias: str | None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        addition_accepts: BasePattern[Any, TInput1, Any],
        previous: BasePattern[TInput4 | TInput1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput4],
            TOrigin1 | None,
        ],
        alias: str | None = None,
        origin: None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        addition_accepts: BasePattern[Any, TInput1, Any],
        previous: BasePattern[TInput4 | TInput1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1 | TInput2, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput4],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        addition_accepts: BasePattern[Any, TInput1, Any],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
        ],
        alias: str | None = None,
        origin: None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        addition_accepts: BasePattern[Any, TInput1, Any],
        previous: BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]], TInput1], TOrigin1 | None
            ]
            | None
        ) = None,
        alias: str | None = None,
        accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        previous: (
            BasePattern[TInput1 | TInput3, TInput2, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[TInput1, TInput2, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[TInput3, TInput2, Literal[MatchMode.TYPE_CONVERT]]
        ),
        addition_accepts: BasePattern[Any, TInput3, Any],
        converter: Callable[
            [
                BasePattern[TOrigin1, TInput1 | TInput2 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
                TInput1 | TInput3,
            ],
            TOrigin1 | None,
        ],
        alias: str | None = None,
        origin: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput2 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        previous: (
            BasePattern[TInput1 | TInput3, TInput2, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[TInput1, TInput2, Literal[MatchMode.TYPE_CONVERT]]
            | BasePattern[TInput3, TInput2, Literal[MatchMode.TYPE_CONVERT]]
        ),
        addition_accepts: BasePattern[Any, TInput3, Any],
        converter: (
            Callable[
                [
                    BasePattern[TOrigin1, TInput1 | TInput2 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
                    TInput1 | TInput3,
                ],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        accepts: type[TInput1],
        previous: (
            BasePattern[TInput1 | TInput3, TInput1 | TInput3, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TInput3, TInput3, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]]
        ),
        addition_accepts: BasePattern[Any, TInput3, Any],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1 | TInput3, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput3],
            TOrigin1 | None,
        ],
        origin: None = None,
        alias: str | None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1 | TInput3, Literal[MatchMode.TYPE_CONVERT]],
        *,
        mode: Literal[MatchMode.TYPE_CONVERT],
        origin: type[TOrigin1],
        accepts: type[TInput1],
        previous: (
            BasePattern[TInput1 | TInput3, TInput1 | TInput3, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TInput3, TInput3, Literal[MatchMode.VALUE_OPERATE]]
            | BasePattern[TInput1, TInput1, Literal[MatchMode.VALUE_OPERATE]]
        ),
        addition_accepts: BasePattern[Any, TInput3, Any],
        converter: (
            Callable[
                [BasePattern[TOrigin1, TInput1 | TInput3, Literal[MatchMode.TYPE_CONVERT]], TInput1 | TInput3],
                TOrigin1 | None,
            ]
            | None
        ) = None,
        alias: str | None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TOrigin1, Literal[MatchMode.VALUE_OPERATE]],
        *,
        mode: Literal[MatchMode.VALUE_OPERATE],
        origin: type[TOrigin1],
        converter: Callable[
            [BasePattern[TOrigin1, TOrigin1, Literal[MatchMode.VALUE_OPERATE]], TOrigin1], TOrigin1 | None
        ],
        alias: str | None = None,
        previous: BasePattern[TOrigin1, TOrigin1, Literal[MatchMode.VALUE_OPERATE]] | None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TInput1, Literal[MatchMode.VALUE_OPERATE]],
        *,
        mode: Literal[MatchMode.VALUE_OPERATE],
        origin: type[TOrigin1],
        previous: BasePattern[TOrigin1, TInput1, Literal[MatchMode.TYPE_CONVERT]],
        converter: Callable[
            [BasePattern[TOrigin1, TInput1, Literal[MatchMode.VALUE_OPERATE]], TOrigin1],
            TOrigin1 | None,
        ],
        alias: str | None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    @overload
    def __init__(
        self: BasePattern[TOrigin1, TOrigin1 | TInput2, Literal[MatchMode.VALUE_OPERATE]],
        *,
        mode: Literal[MatchMode.VALUE_OPERATE],
        origin: type[TOrigin1],
        previous: BasePattern[TOrigin1, TOrigin1 | TInput2, Literal[MatchMode.TYPE_CONVERT]],
        converter: Callable[
            [BasePattern[TOrigin1, TOrigin1 | TInput2, Literal[MatchMode.VALUE_OPERATE]], TOrigin1],
            TOrigin1 | None,
        ],
        alias: str | None = None,
        accepts: None = None,
        addition_accepts: None = None,
        validators: list[Callable[[TOrigin1], bool]] | None = None,
    ): ...
    def refresh(self): ...
    def __calc_hash__(self): ...
    def __calc_repr__(self): ...
    def __calc_eq__(self, other): ...
    def __repr__(self): ...
    def __str__(self): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    @staticmethod
    def of(unit: type[TOrigin1]) -> DirectTypePattern[TOrigin1]:
        """提供 Type[DataUnit] 类型的构造方法"""
        ...

    @staticmethod
    def on(obj: TOrigin1) -> DirectPattern[TOrigin1]:
        """提供 DataUnit 类型的构造方法"""
        ...

    @staticmethod
    def to(content: Any) -> BasePattern:
        """便捷的使用 type_parser 的方法"""
        ...

    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.KEEP]], input_: TInput1
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.KEEP]], input_: T
    ) -> ValidateResult[T, Literal[ResultFlag.ERROR]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.VALUE_OPERATE]], input_: TInput1
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.VALUE_OPERATE]], input_: T
    ) -> ValidateResult[T, Literal[ResultFlag.ERROR]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, TMM], input_: TInput1
    ) -> (
        ValidateResult[TOrigin, Literal[ResultFlag.VALID]]
        | ValidateResult[TOrigin, Literal[ResultFlag.ERROR]]
    ): ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, TMM], input_: T
    ) -> ValidateResult[T, Literal[ResultFlag.ERROR]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.KEEP]], input_: TInput1, default: Any
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput, Literal[MatchMode.KEEP]], input_: Any, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.VALUE_OPERATE]], input_: TInput1, default: Any
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, Literal[MatchMode.VALUE_OPERATE]], input_: Any, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, TMM], input_: TInput1, default: Any
    ) -> (
        ValidateResult[TOrigin, Literal[ResultFlag.VALID]]
        | ValidateResult[TOrigin, Literal[ResultFlag.ERROR]]
    ): ...
    @overload
    def validate(
        self: BasePattern[TOrigin, TInput1, TMM], input_: Any, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...
    def match(self, input_: Any) -> TOrigin: ...
    def copy(self) -> BasePattern[TOrigin, TInput, TMM]: ...
    def __rrshift__(
        self, other: T
    ) -> ValidateResult[T, Literal[ResultFlag.VALID]] | ValidateResult[T, Literal[ResultFlag.ERROR]]: ...
    def __rmatmul__(self, other) -> Self: ...
    def __matmul__(self, other) -> Self: ...
    def __or__(
        self, other: BasePattern[TOrigin1, TInput2, Any]
    ) -> BasePattern[TOrigin1 | TOrigin, TInput2 | TInput, TMM]: ...
